#include <iostream>
#include <libgen.h>

#include "config.h"
#include "support/searchpath.h"  // For SEARCHPATH_SEPARATOR
#include "support/util.h"

#include "argyll_testchartgenerator.h"

#include "gettext.h"
#define _(x) gettext(x)

using namespace std;

// Builds an argument string suitable for Argyll's targen command, specifying colorants.
// BEWARE - Argyll has no support as yet for differentiating between Matte Black and Photo Black,
// so you shouldn't attempt to generate target featuring both types of ink at once.
// Instead, do several independent passes.
//
// EXPERIMENTAL FIX FOR ABOVE ISSUE - use "medium" black instead of Matte Black

string *Argyll_TestChartGenerator::BuildTargenColorants(GPColorantList *colorants)
{
	cerr << "In BuildArgyllColorants" << endl;
	string *result=new string;
	for(int i=0;i<colorants->GetColorantCount();++i)
	{
		GPColorant *col=(GPColorant *)(*colorants)[i];
		if(col && col->GetEnabled())
		{
			cerr << "Checking colorant: " << col->GetName() << endl;
			if(strcasecmp(col->GetName(),"Cyan")==0)
				*result+="-D1 ";
			else if(strcasecmp(col->GetName(),"Magenta")==0)
				*result+="-D2 ";
			else if(strcasecmp(col->GetName(),"Yellow")==0)
				*result+="-D3 ";
			else if(strcasecmp(col->GetName(),"Black")==0)
				*result+="-D4 ";
			else if(strcasecmp(col->GetName(),"Matte Black")==0)
				*result+="-D17 ";
			else if(strcasecmp(col->GetName(),"Photo Black")==0)
				*result+="-D4 ";
			else if(strcasecmp(col->GetName(),"Orange")==0)
				*result+="-D5 ";
			else if(strcasecmp(col->GetName(),"Red")==0)
				*result+="-D6 ";
			else if(strcasecmp(col->GetName(),"Green")==0)
				*result+="-D7 ";
			else if(strcasecmp(col->GetName(),"Blue")==0)
				*result+="-D8 ";
			else if(strcasecmp(col->GetName(),"White")==0)
				*result+="-D9 ";
			else if(strcasecmp(col->GetName(),"Light Cyan")==0)
				*result+="-D10 ";
			else if(strcasecmp(col->GetName(),"Light Magenta")==0)
				*result+="-D11 ";
			else if(strcasecmp(col->GetName(),"Light yellow")==0)
				*result+="-D12 ";
			else if(strcasecmp(col->GetName(),"Light Black")==0)
				*result+="-D13 ";
			else if(strcasecmp(col->GetName(),"Light Light Black")==0)
				*result+="-D18 ";
			else
				cerr << "Unknown colour: " << col->GetName() << endl;
		}
	}
	return(result);
}


Argyll_TestChartGenerator::Argyll_TestChartGenerator() : ArgyllSupport()
{
}


Argyll_TestChartGenerator::~Argyll_TestChartGenerator()
{
}


TI1File *Argyll_TestChartGenerator::CreateRawChannelLinChart(const char *filenameroot,GPColorantList *colorants,int stepsperchannel,Progress *p)
{
	TI1File *result=NULL;
	if(stepsperchannel>9999)
		throw "Too many steps per channel!";
	string *cols=BuildTargenColorants(colorants);
	const char *fmt="%s %s -l100 -f0 -m0 -s%d %s";
#ifdef WIN32
	char *tg=SearchPaths("targen.exe");
#else
	char *tg=SearchPaths("targen");
#endif
	if(!tg)
		throw _("Can't find targen!");
	char *cmd=(char *)malloc(strlen(filenameroot)+strlen(tg)+strlen(fmt)+cols->length()+2);	// Allows 6 chars for %d
	sprintf(cmd,fmt,tg,cols->c_str(),stepsperchannel,filenameroot);
	free(tg);
	delete cols;

	if(p)
		p->SetMessage(_("Generating TI1 file..."));
	cerr << "Calling command: " << cmd << endl;
	if(CallArgyll(cmd,p))
	{
		char *tmpfn=SafeStrcat(filenameroot,".ti1");
		cerr << "Temp filename is: " << tmpfn << endl;
		result=new TI1File(tmpfn);
		free(tmpfn);
	}
	free(cmd);
	return(result);
}


Argyll_TestChart_TIFFs *Argyll_TestChartGenerator::RenderTIFFs(const char *filenameroot,ArgyllPaperSize &size,ArgyllInstrument &inst,int res,int rnd, Progress *p)
{
	DeleteTIFFs(filenameroot);
	Argyll_TestChart_TIFFs *result=NULL;
	const char *fmt="%s -T %d -R %d %s %s %s";
#ifdef WIN32
	char *tg=SearchPaths("printtarg.exe");
#else
	char *tg=SearchPaths("printtarg");
#endif
	if(!tg)
		throw _("Can't find targen!");
	char *sizearg=size.GetPrinttargArgs();
	char *instarg=inst.GetPrinttargArgs();
	char *cmd=(char *)malloc(strlen(filenameroot)+strlen(tg)+strlen(fmt)+strlen(sizearg)+24); // Need headroom for %d arguments
	sprintf(cmd,fmt,tg,res,rnd,instarg,sizearg,filenameroot);
	free(tg);
	free(instarg);
	free(sizearg);

	if(p)
		p->SetMessage(_("Generating TIFFs from TI1 file..."));
	cerr << "Calling command: " << cmd << endl;
	bool success=CallArgyll(cmd,p);
	free(cmd);
	if(success)
		result=new Argyll_TestChart_TIFFs(filenameroot);
	return(result);
}


void Argyll_TestChartGenerator::DeleteTIFFs(const char *filenameroot)
{
	Argyll_TestChart_TIFFs tiffs(filenameroot);
	char *fn=NULL;
	while((fn=tiffs.GetFirstFilename()))
	{
		cerr << "Deleting " << fn << endl;
		remove(fn);
		free(fn);
	}
}


////////////////////////////////////////////////////////////////////////////////////////////////////


Argyll_TestChart_TIFFs::Argyll_TestChart_TIFFs(const char *filenameroot)
	: fnroot(NULL), searchdirectory(NULL)
{
	fnroot=strdup(filenameroot);
}

Argyll_TestChart_TIFFs::~Argyll_TestChart_TIFFs()
{
	if(searchdirectory)
		closedir(searchdirectory);
	searchdirectory=NULL;
	if(fnroot)
		free(fnroot);
}

char *Argyll_TestChart_TIFFs::GetFirstFilename()
{
	if(searchdirectory)
		closedir(searchdirectory);
	searchdirectory=NULL;
	char *tmp=strdup(fnroot);
	searchdirectory=opendir(dirname(tmp));
	free(tmp);

	return(GetNextFilename());
}

char *Argyll_TestChart_TIFFs::GetNextFilename()
{
	if(!searchdirectory)
		return(NULL);
	struct dirent *de=NULL;
	char *bname=basename(fnroot);
	while(!de)
	{
		de=readdir(searchdirectory);
		if(!de)
			return(NULL);
		if(strncmp(bname,de->d_name,strlen(bname))==0)
		{
			if(strlen(de->d_name)>4)
			{
				if(strcasecmp(de->d_name+strlen(de->d_name)-4,".tif")==0)
				{
					char *tmp=strdup(fnroot);
					char *dir=dirname(tmp);
					char *result=(char *)malloc(strlen(de->d_name)+strlen(dir)+2);
					sprintf(result,"%s%c%s",dir,SEARCHPATH_SEPARATOR,de->d_name);
					free(tmp);
					return(result);
				}
			}
		}
		de=NULL;
	}
	return(NULL);
}

