#ifndef LINPROJECT_H
#define LINPROJECT_H

#include "config.h" // For HAVE_LIBPTHREADS define

#include "support/configdb.h"
#include "support/searchpath.h"
#include "support/progress.h"
#include "gp_cppsupport/printoutput.h"
#include "gp_cppsupport/gprintersettings.h"
#include "linearizedprinter.h"

#include "argyll_testchartgenerator.h"

#include "lincore.h"

// LinProject provides a base class for project files for the Linearization tools.
// Inherits ConfigFile, and has PrintOutput and GPrinterSettings menbers, which
// between them can track printer queue and driver settings.

class LinProject : public ConfigFile, public ConfigDB
{
	public:
	LinProject(LinearizationCore &core,const char *filename=NULL);
	virtual ~LinProject();
	virtual bool Load();
	virtual bool Save();
	virtual bool Import(const char *filename);
	virtual void SetFilename(const char *filename);
	virtual const char *GetFilename();
	virtual const char *GetTempDir();
	virtual char *GetTempRoot(const char *id, const char *ext=NULL);	// Utility function - returns a variation
																		// on the temp base filename, with ID
																		// as a suffix - ie. "/path/to/project_temp/project_ID" 

	// Utility function to get and set the Gutenprint InputImageType paramter.
	const char *GetChannelMode();
	void SetChannelMode(const char *mode);

	// High-level chart generation function - gets all the parameters from the project's ConfigDB
	// then uses the Argyll_TestChartGenerator inherited by core to generate the actual chart.
	// Also renders TIFFs from the generated chart.
	virtual void CreateRawChannelLinChart(const char *id="RawLin",Progress *p=NULL);
	virtual TI1File *GetRawChannelLinChart(const char *id="RawLin");
	virtual Argyll_TestChart_TIFFs *GetRawChannelLinTIFFs(const char *id="RawLin");

	// Utility function to create a mapping between a raw DeviceNColorantList and the colorants within a .ti1 file.
	// The array of ints which is returned must be delete[]ed when no longer required.
	virtual int *CreateRawChannelMap(DeviceNColorantList &colorants,TI1File &ti1);

	// Routine to print an image
	// Automatically handles reinterpreting DeviceN data for CMYK mode (FIXME - and RGB data...)
	virtual void Print(ImageSource *src,int left=0,int top=0,bool bypasslinearization=false);

	PrintOutput printoutput;	// Less confusing to make these members than deal with "diamond inheritance".
	LinearizedPrinter gprinter;
	LinearizationCore &core;
	protected:
	char *filename;
	char *tempdirname;
	char *temproot;
	static ConfigTemplate Template[];
};

#endif

