#ifndef PROJECTGUI_H
#define PROJECTGUI_H

#include <gtk/gtk.h>
#include "support/simplecombo.h"
#include "stepselector.h"

#include "linproject.h"
#include "lincore.h"

class StepWidget;
class ProjectGUI : public LinProject
{
	public:
	ProjectGUI(LinearizationCore &core,const char *filename=NULL);
	GtkWidget *window;
	virtual ~ProjectGUI();
	virtual void SetStepWidget(int idx);
	virtual void NextStep();
	virtual GdkPixbuf *GetTestChartPreview(TI1File *ti1,const char *filename);
	virtual void Print(ImageSource *is,int xpos=0,int ypos=0,bool bypasslinearization=false);
	protected:
	StepSelector_StepList steplist;
	GtkUIManager *uimanager;
	GtkWidget *vbox;
	GtkWidget *textview;
	GtkWidget *stepselector;
	StepWidget *stepwidget;
};


class StepWidget
{
	public:
	StepWidget(ProjectGUI *gui) : widget(NULL), modified(false), gui(gui) {}
	virtual ~StepWidget()
	{
		gtk_widget_destroy(widget);
	}
	virtual const char *GetDirections() { return(""); }
	virtual void SaveChanges() {}
	virtual void RevertChanges() {}
	virtual bool IsModified()
	{
		return(modified);
	}
	GtkWidget *widget;
	protected:
	bool modified;
	ProjectGUI *gui;
};


// Wrapper classes for the step selector.
// Generic step class - adds a pure virtual GetWidget() method to the StepSelector_Step class

// The DEFINE_GUI_STEP() macro can be used as a convenience shortcut for subclassing this.

class LinGUIStep : public StepSelector_Step
{
	public:
	LinGUIStep(StepSelector_StepList &header,const char *uiname,ProjectGUI *gui) : StepSelector_Step(header,uiname), gui(gui) {}
	virtual StepWidget *GetWidget()=0;
	protected:
	ProjectGUI *gui;
};

#define PASTE(a,b) a ## b

#define DEFINE_GUI_STEP(x) class PASTE(LinGUIStep_,x) : public LinGUIStep \
{ \
	public: \
	PASTE(LinGUIStep_,x) (StepSelector_StepList &header,char *uiname,ProjectGUI *gui) : LinGUIStep(header,uiname,gui) {} \
	virtual StepWidget *GetWidget() {return(new PASTE(StepWidget_,x)(gui));} \
};


#endif

