#include <iostream>

#include <gtk/gtk.h>

#include "stepwidget_filename.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

using namespace std;

// Filename selection "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window

StepWidget_Filename::StepWidget_Filename(ProjectGUI *gui) : StepWidget(gui)
{
	widget=gtk_vbox_new(FALSE,0);

	filename=gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_SAVE);
	g_signal_connect(G_OBJECT(filename),"file-activated",G_CALLBACK(filename_chosen),this);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(filename),TRUE);
	gtk_box_pack_start(GTK_BOX(widget),filename,TRUE,TRUE,0);
	gtk_widget_grab_focus(filename);
	gtk_widget_show(filename);

	const char *currentfn=gui->GetFilename();
	if(currentfn && strlen(currentfn))
	{
		cerr << "filename is currently set to: " << currentfn << endl;
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filename),currentfn);
		gtk_widget_set_sensitive(GTK_WIDGET(filename),FALSE);
	}
	else
	{
		cerr << "No default filename" << endl;
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(filename),"ProjectName.gplin");
		modified=true;
	}
}


// Text for the Directions box

const char *StepWidget_Filename::GetDirections()
{
	return(_("Please enter a base filename for the project below. "
		"A folder will be created for temporary files, named "
		"<YourChosenFilename>_temp"));
}


void StepWidget_Filename::SaveChanges()
{
	if(modified)
	{
		gchar *newfn=gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filename));
		if(newfn)
		{
			cerr << "Got filename: " << newfn << endl;
			gui->SetFilename(newfn);
			gui->Save();
			g_free(newfn);
		}
	}
	modified=false;
}


// Called when the user hits Enter, or clicks the "Next" button.
void StepWidget_Filename::filename_chosen(GtkWidget *wid,gpointer data)
{
	cerr << "Got filename chosen signal" << endl;
	StepWidget_Filename *step=(StepWidget_Filename *)data;
	step->modified=true;
	step->SaveChanges();

	// Once the filename is chosen, put it in the title bar?

	step->gui->NextStep();
}


void StepWidget_Filename::markmodified(GtkWidget *wid,gpointer data)
{
	StepWidget_Filename *step=(StepWidget_Filename *)data;
	step->modified=true;
}


void StepWidget_Filename::next(GtkWidget *wid,gpointer data)
{
	cerr << "Activating next widget..." << endl;
	StepWidget_Filename *step=(StepWidget_Filename *)data;
	step->gui->NextStep();
}

