#include <iostream>

#include <gtk/gtk.h>

#include "support/generaldialogs.h"
#include "support/util.h"

#include "stpui_widgets/stpui_combo.h"
#include "stpui_widgets/stpui_printerselector.h"
#include "stpui_widgets/stpui_optionbook.h"
#include "gp_cppsupport/printoutputselector.h"

#include "stepwidget_printsetup.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

// Print Setup "step" - self contained class which builds and links all
// the widgets required.  Use the "widget" member to obtain a GtkWidget which can
// be packed into a window

static SimpleComboOption channelmodes[]=
{
//	{"RGB",N_("Composite RGB")},
	{"CMYK",N_("Composite CMYK")},
	{"Raw",N_("Raw printer channels")},
	{NULL,NULL}
};


// Constructor - creates a backup of the current stp_vars so the changes can be
// reverted if desired.
StepWidget_PrintSetup::StepWidget_PrintSetup(ProjectGUI *gui)
	: StepWidget(gui), backupvars(NULL), olddriver(NULL)
{
	backupvars=stp_vars_create_copy(gui->gprinter.stpvars);
	olddriver=SafeStrdup(gui->printoutput.FindString("Driver"));

	widget=gtk_vbox_new(FALSE,0);

	GtkWidget *vbox=gtk_vbox_new(FALSE,0);

	printoutput=printoutputselector_new(&gui->printoutput);
	g_object_ref(G_OBJECT(printoutput));
	gtk_box_pack_start(GTK_BOX(vbox),printoutput,FALSE,FALSE,0);
	gtk_widget_show(printoutput);

	GtkWidget *tmp=gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox),tmp,FALSE,FALSE,8);
	gtk_widget_show(tmp);

	GtkWidget *hbox=gtk_hbox_new(FALSE,8);
	gtk_box_pack_start(GTK_BOX(vbox),hbox,FALSE,FALSE,8);
	gtk_widget_show(hbox);

	tmp=gtk_label_new(_("Channels:"));
	gtk_box_pack_start(GTK_BOX(hbox),tmp,FALSE,FALSE,4);
	gtk_widget_show(tmp);

	channelmode=simplecombo_new(channelmodes);
	simplecombo_set(SIMPLECOMBO(channelmode),stp_get_string_parameter(gui->gprinter.stpvars,"InputImageType"));
	gtk_box_pack_start(GTK_BOX(hbox),channelmode,TRUE,TRUE,4);
	gtk_widget_show(channelmode);

	custompage.name=_("Output");
	custompage.widget=vbox;

	optionbook=stpui_optionbook_new(gui->gprinter.stpvars,&custompage,1);
	gtk_box_pack_start(GTK_BOX(widget),optionbook,TRUE,TRUE,0);
	gtk_widget_show(optionbook);

	g_signal_connect(G_OBJECT(printoutput),"changed",G_CALLBACK(driver_changed),this);
	g_signal_connect(G_OBJECT(optionbook),"changed",G_CALLBACK(optionbook_changed),this);
	g_signal_connect(G_OBJECT(channelmode),"changed",G_CALLBACK(optionbook_changed),this);
}


// Destructor  - prompts the user if called while the "modified" flag - if the user	
// chooses to back out of the changes, restors the stp_vars from the backup made earlier.
StepWidget_PrintSetup::~StepWidget_PrintSetup()
{
	if(backupvars)
		stp_vars_destroy(backupvars);
	if(olddriver)
		free(olddriver);
}


// Return text for the instructions box.
const char *StepWidget_PrintSetup::GetDirections()
{
	return(_("Please choose a printer queue, and adjust the printer settings below.\n"
								"You can also import print settings from a PhotoPrint preset or another\n"
								"GPLin project using the File menu"));
}


// If we're asked to save the changes, copy data from the "live"
// printer class to the config file, then save.
void StepWidget_PrintSetup::SaveChanges()
{
	const char *opt=simplecombo_get(SIMPLECOMBO(channelmode));
	stp_set_string_parameter(gui->gprinter.stpvars,"InputImageType",opt);
	simplecombo_set(SIMPLECOMBO(channelmode),stp_get_string_parameter(gui->gprinter.stpvars,"InputImageType"));
	gui->printoutput.QueuesToDB();
	gui->Save();
	modified=false;
}


// If we're asked to revert the changes, copy from the configdb to
// the live printer class, and copy the backup printer vars
// over the current settings.
void StepWidget_PrintSetup::RevertChanges()
{
	gui->printoutput.DBToQueues();
	stp_vars_copy(gui->gprinter.stpvars,backupvars);
	gui->printoutput.SetString("Driver",olddriver);
	modified=false;
}


// We catch this signal only to mark the modified flag.
void StepWidget_PrintSetup::optionbook_changed(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintSetup *step=(StepWidget_PrintSetup *)data;
	cerr << "OptionBook modified - setting modified flag" << endl;
	step->modified=true;
}


// If the driver changes, we are responsible for informing the optionbook
// of that fact.  We also set the modified flag.
void StepWidget_PrintSetup::driver_changed(GtkWidget *wid,gpointer data)
{
	StepWidget_PrintSetup *step=(StepWidget_PrintSetup *)data;
	PrintOutput &po=step->gui->printoutput;

	cerr << "Driver changed - setting modified flag" << endl;
	step->modified=true;

	if(!(step->gui->gprinter.SetDriver(po.FindString("Driver"))))
	{
		cerr << "Setting driver failed - reverting to default" << endl;
		po.SetString("Driver",DEFAULT_PRINTER_DRIVER);
		step->gui->gprinter.SetDriver(po.FindString("Driver"));
		printoutputselector_refresh(PRINTOUTPUTSELECTOR(step->printoutput));
	}
	stpui_optionbook_rebuild(STPUI_OPTIONBOOK(step->optionbook));	
}

